unit UFormMain;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls,
  StdCtrls, ExtCtrls, UMachine, UInstructionWriter;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    ListViewMemory: TListView;
    ListViewRegisters: TListView;
    Memo1: TMemo;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: char);
    procedure FormShow(Sender: TObject);
    procedure ListViewMemoryData(Sender: TObject; Item: TListItem);
    procedure ListViewRegistersData(Sender: TObject; Item: TListItem);
    procedure Memo1KeyPress(Sender: TObject; var Key: char);
    procedure Timer1Timer(Sender: TObject);
  private
    KeyInputBuffer: array of Char;
    procedure ReloadMemoryDump;
    procedure ReloadRegisterDump;
    function CpuInput(Port: T): T;
    procedure CpuOutput(Port, Value: T);
  public
    Cpu: TCPU;
    InstructionWriter: TInstructionWriter;
  end;

var
  Form1: TForm1;

const
  ItemsPerLine = 16;


implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormShow(Sender: TObject);
var
  LabelStart: Integer;
begin
  with InstructionWriter do begin
    {AddNop;
    AddLdc(1, $ffff);
    AddLd(0, 1);
    AddLdc(2, $10);
    AddStm(2, 0);
    }

    AddLdc(2, 0);
    AddLdc(0, 0);
    LabelStart := IP;
    AddIn(1, 0);
    AddOut(0, 1);
    AddInc(2);
    AddJr(LabelStart);
    AddHalt;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  InstructionWriter.Free;
  Cpu.Free;
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: char);
begin
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Cpu := TCPU.Create(nil);
  Cpu.OnInput := CpuInput;
  Cpu.OnOutput := CpuOutput;
  InstructionWriter := TInstructionWriter.Create;
  InstructionWriter.Cpu := Cpu;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Cpu.Start;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  Cpu.Stop;
end;

procedure TForm1.ListViewMemoryData(Sender: TObject; Item: TListItem);
var
  Line: string;
  I: Integer;
begin
  if Item.Index < Length(Cpu.Memory) div ItemsPerLine then begin
    Line := '';
    for I := 0 to ItemsPerLine - 1 do
      Line := Line + IntToHex(Cpu.Memory[Item.Index * ItemsPerLine + I], 2) + ' ';
    Item.Caption := IntToHex(Item.Index * ItemsPerLine, 8);
    Item.SubItems.Add(Line);
  end;
end;

procedure TForm1.ListViewRegistersData(Sender: TObject; Item: TListItem);
begin
  if Item.Index < Length(Cpu.Registers) then begin
    Item.Caption := 'R' + IntToStr(Item.Index);
    Item.SubItems.Add(IntToHex(Cpu.Registers[Item.Index], 8));
  end;
end;

procedure TForm1.Memo1KeyPress(Sender: TObject; var Key: char);
begin
  SetLength(KeyInputBuffer, Length(KeyInputBuffer) + 1);
  KeyInputBuffer[High(KeyInputBuffer)] := Key;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  ReloadMemoryDump;
  ReloadRegisterDump;
end;

procedure TForm1.ReloadMemoryDump;
begin
  ListViewMemory.Items.Count := Length(Cpu.Memory) div ItemsPerLine;
  ListViewMemory.Refresh;
end;

procedure TForm1.ReloadRegisterDump;
begin
  ListViewRegisters.Items.Count := Length(Cpu.Registers);
  ListViewRegisters.Refresh;
end;

function TForm1.CpuInput(Port: T): T;
begin
  case Port of
    0: begin
      while (Length(KeyInputBuffer) = 0) and not Cpu.Terminated do begin
        Sleep(100);
        Application.ProcessMessages;
      end;
      if Length(KeyInputBuffer) > 0 then begin
        Result := Ord(KeyInputBuffer[0]);
        Move(KeyInputBuffer[1], KeyInputBuffer[0], Length(KeyInputBuffer) - 1);
        SetLength(KeyInputBuffer, Length(KeyInputBuffer) - 1);
      end else Result := 0;
    end;
  end;
end;

procedure TForm1.CpuOutput(Port, Value: T);
begin
  case Port of
    0: Memo1.Lines.Text := Memo1.Lines.Text + Char(Value);
  end;
end;

end.

