unit UInstructionWriter;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, UMachine;

type
  { TInstructionWriter }

  TInstructionWriter = class
  private
  public
    Cpu: TCpu;
    IP: T;
    procedure Write(Value: T);
    procedure AddNop;
    procedure AddLd(R1, R2: Integer);
    procedure AddHalt;
    procedure AddLdc(R, Value: Integer);
    procedure AddLdm(R1, R2: Integer);
    procedure AddStm(R1, R2: Integer);
    procedure AddNeg(R: Integer);
    procedure AddIn(R1, R2: Integer);
    procedure AddOut(R1, R2: Integer);
    procedure AddJr(Addr: Integer);
    procedure AddInc(R: Integer);
    constructor Create;
  end;


implementation

{ TInstructionWriter }

procedure TInstructionWriter.AddNop;
begin
  Write(T(opNop));
end;

procedure TInstructionWriter.AddLd(R1, R2: Integer);
begin
  Write(T(opLoad));
  Write(R1);
  Write(R2);
end;

procedure TInstructionWriter.AddHalt;
begin
  Write(T(opHalt));
end;

procedure TInstructionWriter.AddLdc(R, Value: Integer);
begin
  Write(T(opLoadConst));
  Write(R);
  Write(Value);
end;

procedure TInstructionWriter.AddLdm(R1, R2: Integer);
begin
  Write(T(opLoadMem));
  Write(R1);
  Write(R2);
end;

procedure TInstructionWriter.AddStm(R1, R2: Integer);
begin
  Write(T(opStoreMem));
  Write(R1);
  Write(R2);
end;

procedure TInstructionWriter.AddNeg(R: Integer);
begin
  Write(T(opNeg));
  Write(R);
end;

procedure TInstructionWriter.AddIn(R1, R2: Integer);
begin
  Write(T(opInput));
  Write(R1);
  Write(R2);
end;

procedure TInstructionWriter.AddOut(R1, R2: Integer);
begin
  Write(T(opOutput));
  Write(R1);
  Write(R2);
end;

procedure TInstructionWriter.AddJr(Addr: Integer);
begin
  Write(T(opJumpRel));
  Write(Addr - IP - 1);
end;

procedure TInstructionWriter.AddInc(R: Integer);
begin
  Write(T(opInc));
  Write(R);
end;

constructor TInstructionWriter.Create;
begin
  IP := 0;
  Cpu := nil;
end;

procedure TInstructionWriter.Write(Value: T);
begin
  Cpu.Memory[IP] := Value;
  Inc(IP);
end;


end.

